==> tecfy_corev2_package-1.1.66.tar.gz: lib/models/entity/property.model.dart <==
part of tecfy_corev2_client_package;

abstract class Property {
  late String dbName;
  late ValueTypes valueType;
  late PropertyTypes propertyType;

  bool get fullWidth => size == PropertySizes.MaxSize;
  late int width;
  late PropertySizes size;
  late bool newLineAfter = false;

  late bool readOnly = false;

  late String nameAr;
  late String nameEn;

  String? descriptionAr;
  String? descriptionEn;

  /// get the name according to login language
  String? get description => LocalConfigService.isArabic
      ? (descriptionAr ?? descriptionEn)
      : (descriptionEn ?? descriptionAr);

  dynamic value;

  /// get the name according to login language
  String get name => LocalConfigService.isArabic ? nameAr : nameEn;

  /// if this property will show in edit form
  bool isEdit = false;

  /// if this property will show in add new form
  bool isAdd = false;

  /// if this property will show in grid list
  bool isGrid = false;

  /// sort order in the grid
  int gridOrder = 99;

  /// sort order in the form
  int editOrder = 99;

  /// is required value from user
  bool isRequired = false;

  String? validationFormula;
  String? viewFormula;
  Expression? viewFormulaExp;
  List<Variable>? viewFormulaExpVariables;

  String? editTabName;

  Language? language;
  String? defaultString;

  bool isSearchable = false;
  bool isDisplayField = false;

  /// accept multi values
  bool isMultiValue = false;

  // bool isVisable = true;
  /// get the value and bind tp display widget
  Widget getDisplayWidget(Map<String, dynamic> data,
      {TextStyle? textStyle =
          const TextStyle(fontSize: 12, color: Colors.black)});

  /// get the value in a plain text
  String getDisplayText(Map<String, dynamic> data);

  PropertyBase Function(
          Map<String, dynamic>, void Function(Map<String, dynamic>)?)?
      componentBuilder;

  Property(
      {required this.propertyType,
      required this.dbName,
      required this.nameAr,
      required this.nameEn,
      required this.valueType,
      this.isRequired = false,
      this.readOnly = false,
      this.defaultString,
      this.width = 1,
      this.newLineAfter = false,
      this.size = PropertySizes.Default});

  Property.fromMap(Map<String, dynamic> row) {
    dbName = lowerFirst(row['dbName']) ?? '';

    valueType = row['valueType'] is num
        ? ValueTypes.values[row['valueType']]
        : ValueTypes.values.firstWhere((element) =>
            element.toString() ==
            'ValueTypes.' + row['valueType']); // ?? ValueTypes
    // .Text;
    propertyType = row['propertyType'] is num
        ? PropertyTypes.values[row['propertyType']]
        : PropertyTypes.values.firstWhere(
            (element) =>
                element.toString() == 'PropertyTypes.' + row['propertyType'],
            orElse: () => PropertyTypes.Text,
          ); // ?? PropertyTypes.Text;
    //fullWidth = row['fullWidth'] ?? false;
    width = row['width'] ?? 1;
    newLineAfter = row['newLineAfter'] ?? false;
    size = row['size'] == null
        ? PropertySizes.Default
        : PropertySizes.values[row['size']]; //?? PropertySizes.Default;

    if (BaseModel.getBoolEx(row['fullWidth'])) size = PropertySizes.MaxSize;
    readOnly = row['readOnly'] ?? false;

    nameAr = HelperModel.getName(row, 'name', 'ar');
    nameEn = HelperModel.getName(row, 'name', 'en');
    descriptionAr = HelperModel.getName(row, 'description', 'ar');
    descriptionEn = HelperModel.getName(row, 'description', 'en');
    value = row['value'];
    isEdit = row['isEdit'] ?? false;
    isAdd = row['isAdd'] ?? isEdit;
    isMultiValue = row['isMultiValue'] ?? false;
    isGrid = row['isGrid'] ?? false;
    gridOrder = row['gridOrder'] ?? 99;
    editOrder = row['editOrder'] ?? 99;

    isRequired = row['isRequired'] ?? false;

    editTabName = row['editTabName'];

    language = Language.values[row['language'] ?? 0];
    if (language == null) language = Language.notSelected;

    defaultString = row['default'] ?? '';
    isSearchable = row['isSearchable'] ?? false;
    isDisplayField = row['isDisplayField'] ?? false;

    validationFormula = row['validationFormula'] ?? '';
    viewFormula = row['viewFormula'] ?? '';

    if (readOnly || !isEdit) isRequired = false;

    // if (viewFormula.isNotEmpty) {
    //   var tempList = viewFormula.split(',');
    //   var formula = tempList[0];
    //   var viewFormulaParam = tempList.sublist(1);
    //   var parser = Parser();
    //   viewFormulaExp = parser.parse(formula.replaceAll('e', 'c')); //'(x^2 + cos(y)) / 3'
    //   viewFormulaExpVariables = viewFormulaParam.map((d) => Variable(d.replaceAll('e', 'c'))).toList();
    // }
  }

  static Property? getProperty(Map<String, dynamic> row) {
    PropertyTypes propertyType = row['propertyType'] is num
        ? PropertyTypes.values[row['propertyType']]
        : PropertyTypes.values.firstWhere(
            (element) =>
                element.toString() == 'PropertyTypes.' + row['propertyType'],
            orElse: () => PropertyTypes.Text,
          );
    switch (propertyType) {
      case PropertyTypes.Boolean:
        return PropertyBoolean.fromMap(row);
      case PropertyTypes.Date:
        return PropertyDate.fromMap(row);
      case PropertyTypes.Time:
        return PropertyTime.fromMap(row);
      case PropertyTypes.DateTime:
        return PropertyDateTime.fromMap(row);
      case PropertyTypes.Dropdown:
        return PropertyDropdown.fromMap(row);
      case PropertyTypes.File:
        return PropertyFile.fromMap(row);
      case PropertyTypes.ManyToMany:
        return PropertyManyToMany.fromMap(row);
      case PropertyTypes.MultiDropdown:
      return PropertyMultiDropdown.fromMap(row);
      case PropertyTypes.Number:
        return PropertyNumber.fromMap(row);
      case PropertyTypes.OneToMany:
        return PropertyOneToMany.fromMap(row);
      case PropertyTypes.OtherEntityMany:
        return PropertyOtherEntityMany.fromMap(row);
      case PropertyTypes.Text:
        return PropertyText.fromMap(row);
      case PropertyTypes.Color:
        return PropertyColor.fromMap(row);
      case PropertyTypes.MapArea:
        return PropertyMapArea.fromMap(row);
      case PropertyTypes.MarkImage:
        return PropertyMarkImage.fromMap(row);
      case PropertyTypes.Location:
        return PropertyLocation.fromMap(row);
      case PropertyTypes.Time:
      // return PropertyTime.fromMap(row);
      case PropertyTypes.TextMultiLanguage:
        return PropertyTextMultiLang.fromMap(row);
      case PropertyTypes.FileMulti:
        return PropertyFile.fromMap(row);
      case PropertyTypes.DynamicProperties:
        return PropertyDynamicProperties.fromMap(row);
      case PropertyTypes.Html:
        return PropertyHtml.fromMap(row);
      default:
        return null;
    }
  }

  String? lowerFirst(String? s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String? upperFirst(String? s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toUpperCase()}${s.substring(1)}';
  }

  bool get isImage =>
      this.propertyType == PropertyTypes.File &&
      (this as PropertyFile).isImage == true;

  OperationResult isValid(dynamic data) {
    if (!isAdd || !isEdit || readOnly == true)
      return OperationResult(success: true);
    if ((isRequired && data != null) || (!isRequired)) {
      return OperationResult(success: true);
    } else {
      return OperationResult(
          success: false,
          message: "${AppLocale.translate('Value Required')} ($name)");
    }
  }

  Future<bool> isVisible(Map<String, dynamic> data, Entity? entity) async {
    if (entity == null) return true;
    if (viewFormula != null && (viewFormula ?? '').isNotEmpty) {
      var params = Map<String, dynamic>();
      for (var i in viewFormula?.split(',').sublist(1) ?? []) {
        params[i.replaceAll('.', '')] =
            await ValuesService.getValueByString(i, data, entity.properties);
      }
      return getViewFormulaResult(params);
    }
    return true;
  }

  bool getViewFormulaResult(Map<String, dynamic> vars) {
    try {
      var fString = viewFormula?.split(',').first.replaceAll('.', '') ?? '';
      var f = Expression.parse(fString);
      final evaluator = const ExpressionEvaluator();
      var r = evaluator.eval(f, vars);
      return r;
    } catch (err) {
      print('XXXXXXXXXX getViewFormulaResult $err');
      return false;
    }
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data);

  String? get filterValueField {
    if (propertyType == PropertyTypes.Dropdown) {
      return (this as PropertyDropdown).filterValueField;
    } else if (propertyType == PropertyTypes.ManyToMany) {
      return (this as PropertyManyToMany).filterValueField;
    }
    return "";
  }

  Map<String, dynamic> toJson() {
    return {
      'dbName': dbName,
      'name': {'ar': nameAr, 'en': nameEn},
      'propertyType': propertyType.index,
      'valueType': valueType.index,
      'readOnly': readOnly,
      'isRequired': isRequired,
    };
  }
}

==> tecfy_corev2_package-1.1.71.tar.gz: lib/models/entity/property.model.dart <==
part of tecfy_corev2_client_package;

abstract class Property {
  late String dbName;
  late ValueTypes valueType;
  late PropertyTypes propertyType;

  bool get fullWidth => size == PropertySizes.MaxSize;
  late int width;
  late PropertySizes size;
  late bool newLineAfter = false;

  late bool readOnly = false;
  late bool readOnlyOnEdit = false;

  late String nameAr;
  late String nameEn;

  String? descriptionAr;
  String? descriptionEn;

  /// get the name according to login language
  String? get description => LocalConfigService.isArabic
      ? (descriptionAr ?? descriptionEn)
      : (descriptionEn ?? descriptionAr);

  dynamic value;

  /// get the name according to login language
  String get name => LocalConfigService.isArabic ? nameAr : nameEn;

  /// if this property will show in edit form
  bool isEdit = false;

  /// if this property will show in add new form
  bool isAdd = false;

  /// if this property will show in grid list
  bool isGrid = false;

  /// sort order in the grid
  int gridOrder = 99;

  /// sort order in the form
  int editOrder = 99;

  /// is required value from user
  bool isRequired = false;

  String? validationFormula;
  String? viewFormula;
  Expression? viewFormulaExp;
  List<Variable>? viewFormulaExpVariables;

  String? editTabName;

  Language? language;
  String? defaultString;

  bool isSearchable = false;
  bool isDisplayField = false;

  /// accept multi values
  bool isMultiValue = false;

  // bool isVisable = true;
  /// get the value and bind tp display widget
  Widget getDisplayWidget(Map<String, dynamic> data,
      {TextStyle? textStyle =
          const TextStyle(fontSize: 12, color: Colors.black)});

  /// get the value in a plain text
  String getDisplayText(Map<String, dynamic> data);

  PropertyBase Function(
          Map<String, dynamic>, void Function(Map<String, dynamic>)?)?
      componentBuilder;

  Property(
      {required this.propertyType,
      required this.dbName,
      required this.nameAr,
      required this.nameEn,
      required this.valueType,
      this.isRequired = false,
      this.readOnly = false,
      this.defaultString,
      this.width = 1,
      this.newLineAfter = false,
      this.size = PropertySizes.Default});

  Property.fromMap(Map<String, dynamic> row) {
    dbName = lowerFirst(row['dbName']) ?? '';

    valueType = row['valueType'] is num
        ? ValueTypes.values[row['valueType']]
        : ValueTypes.values.firstWhere((element) =>
            element.toString() ==
            'ValueTypes.' + row['valueType']); // ?? ValueTypes
    // .Text;
    propertyType = row['propertyType'] is num
        ? PropertyTypes.values[row['propertyType']]
        : PropertyTypes.values.firstWhere(
            (element) =>
                element.toString() == 'PropertyTypes.' + row['propertyType'],
            orElse: () => PropertyTypes.Text,
          ); // ?? PropertyTypes.Text;
    //fullWidth = row['fullWidth'] ?? false;
    width = row['width'] ?? 1;
    newLineAfter = row['newLineAfter'] ?? false;
    size = row['size'] == null
        ? PropertySizes.Default
        : PropertySizes.values[row['size']]; //?? PropertySizes.Default;

    if (BaseModel.getBoolEx(row['fullWidth'])) size = PropertySizes.MaxSize;
    readOnly = row['readOnly'] ?? false;
    readOnlyOnEdit = row['readOnlyOnEdit'] ?? false;

    nameAr = HelperModel.getName(row, 'name', 'ar');
    nameEn = HelperModel.getName(row, 'name', 'en');
    descriptionAr = HelperModel.getName(row, 'description', 'ar');
    descriptionEn = HelperModel.getName(row, 'description', 'en');
    value = row['value'];
    isEdit = row['isEdit'] ?? false;
    isAdd = row['isAdd'] ?? isEdit;
    isMultiValue = row['isMultiValue'] ?? false;
    isGrid = row['isGrid'] ?? false;
    gridOrder = row['gridOrder'] ?? 99;
    editOrder = row['editOrder'] ?? 99;

    isRequired = row['isRequired'] ?? false;

    editTabName = row['editTabName'];

    language = Language.values[row['language'] ?? 0];
    if (language == null) language = Language.notSelected;

    defaultString = row['default'] ?? '';
    isSearchable = row['isSearchable'] ?? false;
    isDisplayField = row['isDisplayField'] ?? false;

    validationFormula = row['validationFormula'] ?? '';
    viewFormula = row['viewFormula'] ?? '';

    if (readOnly || !isEdit) isRequired = false;

    // if (viewFormula.isNotEmpty) {
    //   var tempList = viewFormula.split(',');
    //   var formula = tempList[0];
    //   var viewFormulaParam = tempList.sublist(1);
    //   var parser = Parser();
    //   viewFormulaExp = parser.parse(formula.replaceAll('e', 'c')); //'(x^2 + cos(y)) / 3'
    //   viewFormulaExpVariables = viewFormulaParam.map((d) => Variable(d.replaceAll('e', 'c'))).toList();
    // }
  }

  static Property? getProperty(Map<String, dynamic> row) {
    PropertyTypes propertyType = row['propertyType'] is num
        ? PropertyTypes.values[row['propertyType']]
        : PropertyTypes.values.firstWhere(
            (element) =>
                element.toString() == 'PropertyTypes.' + row['propertyType'],
            orElse: () => PropertyTypes.Text,
          );
    switch (propertyType) {
      case PropertyTypes.Boolean:
        return PropertyBoolean.fromMap(row);
      case PropertyTypes.Date:
        return PropertyDate.fromMap(row);
      case PropertyTypes.Time:
        return PropertyTime.fromMap(row);
      case PropertyTypes.DateTime:
        return PropertyDateTime.fromMap(row);
      case PropertyTypes.Dropdown:
        return PropertyDropdown.fromMap(row);
      case PropertyTypes.File:
        return PropertyFile.fromMap(row);
      case PropertyTypes.ManyToMany:
        return PropertyManyToMany.fromMap(row);
      case PropertyTypes.MultiDropdown:
        return PropertyMultiDropdown.fromMap(row);
      case PropertyTypes.Number:
        return PropertyNumber.fromMap(row);
      case PropertyTypes.OneToMany:
        return PropertyOneToMany.fromMap(row);
      case PropertyTypes.OtherEntityMany:
        return PropertyOtherEntityMany.fromMap(row);
      case PropertyTypes.Text:
        return PropertyText.fromMap(row);
      case PropertyTypes.Color:
        return PropertyColor.fromMap(row);
      case PropertyTypes.MapArea:
        return PropertyMapArea.fromMap(row);
      case PropertyTypes.MarkImage:
        return PropertyMarkImage.fromMap(row);
      case PropertyTypes.Location:
        return PropertyLocation.fromMap(row);
      case PropertyTypes.Time:
      // return PropertyTime.fromMap(row);
      case PropertyTypes.TextMultiLanguage:
        return PropertyTextMultiLang.fromMap(row);
      case PropertyTypes.FileMulti:
        return PropertyFile.fromMap(row);
      case PropertyTypes.DynamicProperties:
        return PropertyDynamicProperties.fromMap(row);
      case PropertyTypes.Html:
        return PropertyHtml.fromMap(row);
      default:
        return null;
    }
  }

  String? lowerFirst(String? s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String? upperFirst(String? s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toUpperCase()}${s.substring(1)}';
  }

  bool get isImage =>
      this.propertyType == PropertyTypes.File &&
      (this as PropertyFile).isImage == true;

  OperationResult isValid(dynamic data) {
    if (!isAdd || !isEdit || readOnly == true || readOnlyOnEdit == true)
      return OperationResult(success: true);
    if ((isRequired && data != null) || (!isRequired)) {
      return OperationResult(success: true);
    } else {
      return OperationResult(
          success: false,
          message: "${AppLocale.translate('Value Required')} ($name)");
    }
  }

  Future<bool> isVisible(Map<String, dynamic> data, Entity? entity) async {
    if (entity == null) return true;
    if (viewFormula != null && (viewFormula ?? '').isNotEmpty) {
      var params = Map<String, dynamic>();
      for (var i in viewFormula?.split(',').sublist(1) ?? []) {
        params[i.replaceAll('.', '')] =
            await ValuesService.getValueByString(i, data, entity.properties);
      }
      return getViewFormulaResult(params);
    }
    return true;
  }

  bool getViewFormulaResult(Map<String, dynamic> vars) {
    try {
      var fString = viewFormula?.split(',').first.replaceAll('.', '') ?? '';
      var f = Expression.parse(fString);
      final evaluator = const ExpressionEvaluator();
      var r = evaluator.eval(f, vars);
      return r;
    } catch (err) {
      print('XXXXXXXXXX getViewFormulaResult $err');
      return false;
    }
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data);

  String? get filterValueField {
    if (propertyType == PropertyTypes.Dropdown) {
      return (this as PropertyDropdown).filterValueField;
    } else if (propertyType == PropertyTypes.ManyToMany) {
      return (this as PropertyManyToMany).filterValueField;
    }
    return "";
  }

  Map<String, dynamic> toJson() {
    return {
      'dbName': dbName,
      'name': {'ar': nameAr, 'en': nameEn},
      'propertyType': propertyType.index,
      'valueType': valueType.index,
      'readOnly': readOnly,
      'readOnlyOnEdit': readOnlyOnEdit,
      'isRequired': isRequired,
    };
  }
}
